<?php

namespace HCM\Geral\Services;

use GuzzleHttp\Client;

use HCM\Geral\Repository\OperadorRepository;
use HCM\Geral\Util\ExpectedException;

class HcmServices {
    protected $http;
    protected $url;

    // PHP Server constants
    protected const INTERNAL_SERVER_ERROR = 500;
    protected const PHP_SERVER_ERROR_MESSAGE = 'PHP Error';

    // Response constants
    protected const RESPONSE_BODY_MESSAGES_INDEX = 'messages';
    protected const RESPONSE_BODY_MESSAGES_MESSAGE_INDEX = 'message';
    protected const RESPONSE_BODY_ERROR_INDEX = 'error';
    protected const RESPONSE_BODY_ERRORCODE_INDEX = 'errorCode';
    protected const RESPONSE_BODY_FATAL_ERROR = '<b>Fatal error</b>';
    protected const RESPONSE_UNKNOWN_ERROR = 'Houve um erro ao acessar o webservice.';

    // Intern constants
    protected const CHECKACCESS_METHOD  = 'POST';
    protected const REQUESTTOKEN_METHOD = 'POST';

    public function __construct(
        string $hcmWebServiceUrl,
        OperadorRepository $operadorRepository
    ) {
        $this->operadorRepository = $operadorRepository;
        $this->url = $hcmWebServiceUrl;
        $this->http = new Client([
            'base_uri' => $this->url
        ]);

        if (!isset($_SESSION['HCM_TOKEN'])) {
            $_SESSION['HCM_TOKEN'] = '';
        }

        if (!isset($_SESSION['HCM_SECRET'])) {
            $_SESSION['HCM_SECRET'] = '';
        }
    }

    public function connect(int $nrOrg) {
        if (!$this->checkAccess($nrOrg)) {
            $token = $this->requestToken($nrOrg);
            $_SESSION['HCM_TOKEN'] = $token;
        }
    }

    public function checkAccess(int $nrOrg): bool {
        if ($_SESSION['HCM_TOKEN'] === '') {
            return false;
        }

        $params = [
            'TOKEN' => $_SESSION['HCM_TOKEN'],
            'CLIENT_SECRET' => $_SESSION['HCM_SECRET'],
            'UserData-CdOperador' => 1
        ];

        $response = $this->send('checkAccess', $params, $nrOrg, self::CHECKACCESS_METHOD);

        $access = $response['messages'][0]['type'] === 'M';

        return $access;
    }

    public function requestToken(int $nrOrg): string {
        $params = [
            'CLIENT_ID' => $nrOrg,
            'CLIENT_SECRET' => $_SESSION['HCM_SECRET']
        ];

        $response = $this->send('requestToken', $params, $nrOrg, self::REQUESTTOKEN_METHOD);

        if (isset($response['messages'])) {
            throw new \Exception($response['messages'][0]['message'], self::INTERNAL_SERVER_ERROR);
        }

        return $response['dataset']['token'][0];
    }

    public function send(
        string  $uri,
        array   $params,
                $nrOrg,
                $method             = 'POST',
                $throwIfHasError    = true
    ): array {
        $headers = [];

        if ($_SESSION['HCM_SECRET'] === '') {
            $this->getClientSecret($nrOrg);
        }

        if (!($uri === 'checkAccess' || $uri === 'requestToken')) {
            $this->connect($nrOrg);

            $headers['OAuth-Token'] = $_SESSION['HCM_TOKEN'];
            $headers['OAuth-SecretId'] = $_SESSION['HCM_SECRET'];
        }

        $response = $this->http->request($method, $uri, [
            'json' => $params,
            'headers' => $headers
        ]);

        $content = $response->getBody()->getContents();
        $body = json_decode($content, true);

        // Error when $content has PHP_SERVER_ERROR_MESSAGE
        $bodyHasError = isset($body[self::RESPONSE_BODY_ERROR_INDEX]);

        // Message when $body[self::RESPONSE_BODY_MESSAGES_INDEX] is set
        $bodyHasMessage = isset($body[self::RESPONSE_BODY_MESSAGES_INDEX]);

        if ($throwIfHasError) {
            self::throwIfHasError($content, $uri);
        }

        return $body;
    }

    public function getClientSecret(int $nrOrg) {
        $result = $this->operadorRepository->getClientSecret($nrOrg);
        $clientSecret = $result[0]['CLIENT_SECRET'];
        $_SESSION['HCM_SECRET'] = $clientSecret;

        return $clientSecret;
    }

    public static function throwIfHasError($content, $uri) {
        $body = json_decode($content, true);

        $fromHcmServices = '(HCMSERVICES) ';

        // Error when $content has PHP_SERVER_ERROR_MESSAGE
        $bodyHasError = isset($body[self::RESPONSE_BODY_ERROR_INDEX]);

        // Message when $body[self::RESPONSE_BODY_MESSAGES_INDEX] is set
        $bodyHasMessage = isset($body[self::RESPONSE_BODY_MESSAGES_INDEX]);

        if ($bodyHasError) {
            $errorMessage = $body[self::RESPONSE_BODY_ERROR_INDEX];
            throw new \Exception($fromHcmServices . $errorMessage, self::INTERNAL_SERVER_ERROR);
        }

        if ($bodyHasMessage) {
            if ($uri === 'requestToken' || $uri === 'checkAccess') {
                return;
            }

            $errorMessage = $body[self::RESPONSE_BODY_MESSAGES_INDEX][0]['message'];
            throw new ExpectedException($fromHcmServices . $errorMessage, self::INTERNAL_SERVER_ERROR);
        }

        $hasErrorText =
            strpos($content, self::RESPONSE_BODY_FATAL_ERROR);

        if ($hasErrorText) {
            throw new \Exception($fromHcmServices . $content, self::INTERNAL_SERVER_ERROR);
        }

        // Provavelmente veio de um var_dump
        if (!isset($body['dataset'])) {
            die($content);
        }
    }
}